<?php
/**
 * Plugin Name: Excerpt Editor for Beaver Builder
 * Plugin URI: http://www.particulare.nl
 * Description: The Excerpt Editor addon is build for Beaver Builder to add the possibility to edit the excerpt of a page or post.
 * Version: 1.0.2
 * Author: Jack Krielen
 * Author URI: http://www.jackkrielen.nl
 * Copyright: (c) 2018 Particulare
 * License: GNU General Public License v2.0
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: excerpt-editor-for-beaver-builder
 * Domain Path: /languages
 */

// Exit if accessed directly.
if (!defined('ABSPATH')) {
  exit;
}

final class BB_Addon_excerpt {

  /**
   * Holds the class object.
   *
   * @since 1.0.0
   * @var object
   */
  public static $instance;

  /**
   * Primary class constructor.
   *
   * @since 1.0.0
   */
  public function __construct() {
    /* Constants */
    $this->define_constants();


    /* Hooks */
    $this->init_hooks();

  }

  /**
   * Define BB Addons constants.
   *
   * @since 1.0.0
   * @return void
   */
  private function define_constants() {
    define('BB_excerpt_DIR', plugin_dir_path(__FILE__));
    define('BB_excerpt_URL', plugins_url('/', __FILE__));
    define('BB_excerpt_PATH', plugin_basename(__FILE__));
  }

  /**
   * Initializes actions and filters.
   *
   * @since 1.0.0
   * @return void
   */
  public function init_hooks() {
    add_action('init', array($this, 'load_excerpt'));
    add_action('plugins_loaded', array($this, 'loader'));
    add_action('wp_enqueue_scripts', array($this, 'load_scripts'), 100);
    add_action('admin_notices', array($this, 'admin_notices'));
    add_action('network_admin_notices', array($this, 'admin_notices'));
  }

  /**
   * Include excerpt.
   *
   * @since 1.0.0
   * @return void
   */
  public function load_excerpt() {
    if (class_exists('FLBuilder')) {
      add_filter('fl_builder_ui_bar_buttons', array(
        $this,
        'filter_builder_ui_bar_buttons'
      ));

      // Adds a ajax script.
      FLBuilderAJAX::add_action('save_excerpt_settings', 'BB_Addon_excerpt::save_excerpt_settings', array(
        'settings',
        'post_id'
      ));

      add_filter('fl_builder_ui_js_config', array(
        $this,
        'excerpt_builder_ui_js_config'
      ));

      require_once 'settings/bb-excerpt-form.php';
    }
  }

  /**
   * Include row and column setting extendor.
   *
   * @since 1.0.0
   * @return void
   */
  public function loader() {
    if (class_exists('FLBuilder')) {
      add_filter('fl_builder_custom_fields', array(
        $this,
        'register_fields'
      ));

      $this->load_textdomain();
    }
  }

  /**
   * Register a new "fake" field.
   *
   * @since 1.0.0
   * @param $fields
   * @return array
   */
  public function register_fields($fields) {
    $fields['showexcerpt'] = BB_excerpt_DIR . 'fields/showexcerpt.php';
    return $fields;
  }

  /**
   * New Ajax method to save the excerpt Settings for the post / page
   *
   * @since 1.0.0
   * @param array $settings
   * @param string $post_id
   * @return array
   */
  static public function save_excerpt_settings($settings = array(), $post_id = '') {
    $postarr = array(
      'ID' => $post_id,
      'post_excerpt' => $settings['excerpt']
    );

    wp_update_post($postarr);

    return self::get_post_excerpt($post_id);
  }

  /**
   * Load in excerpt JS.
   *
   * @since 1.0.0
   *
   * @return void
   */
  public function load_scripts() {
    if (class_exists('FLBuilderModel') && FLBuilderModel::is_builder_active()) {
      wp_enqueue_script('bb-excerpt', BB_excerpt_URL . 'assets/js/excerpt.js', array('jquery'), rand(), TRUE);
    }
  }

  /**
   * Admin notices.
   *
   * @since 1.0.0
   *
   * @return void
   */
  public function admin_notices() {
    if (!is_admin()) {
      return;
    }
    else {
      if (!is_user_logged_in()) {
        return;
      }
      else {
        if (!current_user_can('update_core')) {
          return;
        }
      }
    }

    if (!is_plugin_active('bb-plugin/fl-builder.php')) {
      if (!is_plugin_active('beaver-builder-lite-version/fl-builder.php')) {
        echo sprintf('<div class="notice notice-error"><p>%s</p></div>', __('Please install and activate <a href="https://wordpress.org/plugins/beaver-builder-lite-version/" target="_blank">Beaver Builder Lite</a> or <a href="https://www.wpbeaverbuilder.com/pricing/" target="_blank">Beaver Builder Pro / Agency</a> to use excerpt Editor for Beaver Builder.', 'excerpt-editor-for-beaver-builder'));
      }
    }
  }

  /**
   * Get the Excerpt
   * @param string $postid
   * @return array
   */
  static public function get_post_excerpt($postid = '') {
    $post = get_post($postid);
    return array(
      'excerpt' => has_excerpt($postid) ? get_the_excerpt($postid) : '',
      'currentexcerpt' => has_excerpt($postid) ? self::bb_trim_excerpt($post->post_content) : get_the_excerpt($postid)
    );
  }

  /**
   * Add Excerpt to the config
   * @param $array
   * @return mixed
   */
  public function excerpt_builder_ui_js_config($array) {

    $array['excerpt'] = self::get_post_excerpt($array['postId']);
    return $array;
  }


  /**
   * Copy of wp_trim_excerpt, but then allows Text;
   *
   * @param string $text
   * @return mixed|void
   */
  static public function bb_trim_excerpt($text = '') {
    $raw_excerpt = $text;

    $text = strip_shortcodes($text);

    /** This filter is documented in wp-includes/post-template.php */
    $text = apply_filters('the_content', $text);
    $text = str_replace(']]>', ']]&gt;', $text);

    /**
     * Filters the number of words in an excerpt.
     *
     * @since 2.7.0
     *
     * @param int $number The number of words. Default 55.
     */
    $excerpt_length = apply_filters('excerpt_length', 55);
    /**
     * Filters the string in the "more" link displayed after a trimmed excerpt.
     *
     * @since 2.9.0
     *
     * @param string $more_string The string shown within the more link.
     */
    $excerpt_more = apply_filters('excerpt_more', ' ' . '>>');
    $text = wp_trim_words($text, $excerpt_length, $excerpt_more);
    /**
     * Filters the trimmed excerpt string.
     *
     * @since 2.8.0
     *
     * @param string $text The trimmed text.
     * @param string $raw_excerpt The text prior to trimming.
     */
    return apply_filters('wp_trim_excerpt', $text, $raw_excerpt);
  }

  /**
   * Load language files.
   *
   * @since 1.0.0
   * @return void
   */
  public function load_textdomain()
  {
    load_plugin_textdomain( 'excerpt-editor-for-beaver-builder', FALSE, basename( dirname( __FILE__ ) ) . '/languages/' );
  }


  /**
   * Returns the singleton instance of the class.
   *
   * @since 1.0.0
   *
   * @return object The BB_Addon_excerpt object.
   */
  public static function get_instance() {
    if (!isset(self::$instance) && !(self::$instance instanceof BB_Addon_excerpt)) {
      self::$instance = new BB_Addon_excerpt();
    }

    return self::$instance;
  }

  public function filter_builder_ui_bar_buttons($buttons) {
    $buttons['excerpt'] = array(
      'label' => __('Excerpt', 'excerpt-editor-for-beaver-builder')
    );
    return $buttons;
  }

}

// Load the PowerPack class.
$bb_addon_excerpt = BB_Addon_excerpt::get_instance();
